
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

            #include <glib-object.h>


            G_BEGIN_DECLS

/* enumerations from "app-filter.h" */


GType mct_app_filter_oars_value_get_type (void);
#define MCT_TYPE_APP_FILTER_OARS_VALUE (mct_app_filter_oars_value_get_type())

/* enumerations from "manager.h" */


GType mct_manager_get_value_flags_get_type (void);
#define MCT_TYPE_MANAGER_GET_VALUE_FLAGS (mct_manager_get_value_flags_get_type())


GType mct_manager_set_value_flags_get_type (void);
#define MCT_TYPE_MANAGER_SET_VALUE_FLAGS (mct_manager_set_value_flags_get_type())


GType mct_manager_error_get_type (void);
#define MCT_TYPE_MANAGER_ERROR (mct_manager_error_get_type())

G_END_DECLS

/* Generated data ends here */

